/*
-------------------------------------------------------------

		   Multiple Decision Diagrams

			   Interface

-------------------------------------------------------------
*/

/*
-------------------------------------------------------------
	Module 		    :		mdd.h
	Date		    :		12/04/96
	Version		    :		1.10
	Author		    :		Dorel Marius Bozga
-------------------------------------------------------------
*/

#ifndef MDD_H
#define MDD_H

/*
-------------------------------------------------------------
	Types
-------------------------------------------------------------
*/

typedef struct mdd_n{
   	int code; /* variable index, test value, flags */
   	struct mdd_n* low; /* child to variable <= value */
   	struct mdd_n* high; /* child to variable > value */
   	struct mdd_n* next; /* next node in hash table */
   	} mdd;

typedef struct{
      	long allowed; /* initial allowed memory */
   	struct {  /* variable informations */
      		int number; /* the number of variables */
      		int* range; /* the range for each variable */
      		struct {
		        int* low; /* low variable values */
		        int* high; /* high variable values */
		        } buffer; /* buffer for variable values */
      		} variable;
        struct { /* static mdd nodes table */
		mdd* crt; /* current node in table */
		mdd* nodes; /* all mdd nodes are here */
	        } table;
   	struct { /* the internal nodes hash table */
      		int size; /* hash table size */
      		int capacity; /* the max number of nodes */
      		int nodes; /* the crt number of nodes */
      		mdd** entry; /* the entries */
      		} hashtable;
   	struct { /* caches */
      		int size; /* cache size */
      		mdd** and[3]; /* and cache */
		mdd** andex[3]; /* and exist cache */
      		mdd** or[3]; /* or cache */
      		mdd** not[2]; /* not cache */
      		mdd** exist[2]; /* exist cache */
      		mdd** subst[2]; /* subst cache */
      		} cache;
   	struct { /* basic constants */
      		mdd* one; /* one constant */
      		mdd* zero; /* zero constant */
      		} constant;
   	struct { /* user references */
	        mdd** nodes; /* the referenced nodes table */
	        int index; /* first unused position in table */
	        int size; /* current table size */
	        } references;
   	} mdd_manager;

/*
-------------------------------------------------------------
	Exported MDD routines
-------------------------------------------------------------
*/

/* Function prototypes */
#if defined(__STDC__) || defined(__cplusplus)
#define ARGS(args) args
#else
#define ARGS(args) ()
#endif

#ifdef __cplusplus
#define EXTERN extern "C"
#else
#define EXTERN extern
#endif

/*
-------------------------------------------------------------
	MDD Manager Allocation and Destruction
-------------------------------------------------------------
*/

EXTERN mdd_manager* mdd_manager_create ARGS((int, int));
EXTERN void mdd_manager_destroy ARGS((mdd_manager*));
EXTERN void mdd_manager_print_info ARGS((mdd_manager*, FILE*));

/*
-------------------------------------------------------------
	MDD Nodes Basic Creation and Management 
-------------------------------------------------------------
*/

EXTERN mdd* mdd_zero ARGS((mdd_manager*));
EXTERN mdd* mdd_one ARGS((mdd_manager*));

EXTERN void mdd_declare_variable ARGS((mdd_manager*, int, int));
EXTERN mdd* mdd_get_variable ARGS((mdd_manager*, int, int));

EXTERN void mdd_manager_keep ARGS((mdd_manager*, mdd*));
EXTERN void mdd_manager_release ARGS((mdd_manager*, mdd*));

/*
-------------------------------------------------------------
	Operations on MDD's formulas
-------------------------------------------------------------
*/

EXTERN mdd* mdd_not ARGS((mdd_manager*, mdd*));
EXTERN mdd* mdd_or ARGS((mdd_manager*, mdd*, mdd*));
EXTERN mdd* mdd_and ARGS((mdd_manager*, mdd*, mdd*));

EXTERN mdd* mdd_exist ARGS((mdd_manager*, mdd*, int*));
EXTERN mdd* mdd_and_exist ARGS((mdd_manager*, mdd*, mdd*, int*));
EXTERN mdd* mdd_substitute ARGS((mdd_manager*, mdd*, int*));

EXTERN void mdd_print ARGS((mdd_manager*, mdd*, FILE*));
EXTERN void mdd_dump ARGS((mdd_manager*, mdd*, FILE*));

EXTERN int mdd_size ARGS((mdd_manager*, mdd*));
EXTERN double mdd_onset_size ARGS((mdd_manager*, mdd*, int*));


#endif
